@@ -1,6 +1,6 @@
 package App::ansiecho;
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
 
 use v5.14;
 use warnings;
@@ -17,57 +17,61 @@ use Data::Dumper;
 use open IO => 'utf8', ':std';
 use Pod::Usage;
 
-use Moo;
+use Getopt::EX::Hashed;
 
-has debug      => ( is => 'ro' );
-has verbose    => ( is => 'ro', default => 1 );
-has no_newline => ( is => 'ro' );
-has join       => ( is => 'ro' );
-has escape     => ( is => 'ro', default => 1 );
-has rgb24      => ( is => 'ro' );
-has separate   => ( is => 'rw', default => " " );
+has debug      => spec => "      " ;
+has no_newline => spec => " n !  " ;
+has join       => spec => " j !  " ;
+has escape     => spec => " e !  " , default => 1;
+has rgb24      => spec => "   !  " ;
+has separate   => spec => "   =s " , default => " ";
+has help       => spec => " h    " ;
+has version    => spec => " v    " ;
 
-has terminate  => ( is => 'rw', default => "\n" );
-has params     => ( is => 'rw' );
+has '+help' => action => sub {
+    pod2usage
+	-verbose  => 99,
+	-sections => [ qw(SYNOPSIS VERSION) ];
+};
 
-no Moo;
+has '+version' => action => sub {
+    say "Version: $VERSION";
+    exit;
+};
+
+has terminate  => default => "\n";
+has params     => default => [];
+
+no Getopt::EX::Hashed;
 
 use App::ansiecho::Util;
-use Getopt::EX v1.23.2;
+use Getopt::EX v1.24.1;
 use Text::ANSI::Printf 2.01 qw(ansi_sprintf);
 
 use List::Util qw(sum);
 
 sub run {
     my $app = shift;
-    local @ARGV = map { utf8::is_utf8($_) ? $_ : decode('utf8', $_) } @_;
+    local @ARGV = decode_argv @_;
 
     use Getopt::EX::Long qw(GetOptions Configure ExConfigure);
     ExConfigure BASECLASS => [ __PACKAGE__, "Getopt::EX" ];
     Configure qw(bundling no_getopt_compat pass_through);
-    GetOptions($app, make_options "
-	debug
-	verbose    | v !
-	no_newline | n !
-	join       | j !
-	escape     | e !
-	rgb24          !
-	separate       =s
-	") || pod2usage();
+    $app->getopt || pod2usage();
     $app->initialize();
-    $app->params(\@ARGV);
-    print join($app->separate, $app->retrieve()), $app->terminate;
+    $app->{params} = \@ARGV;
+    print join($app->{separate}, $app->retrieve()), $app->{terminate};
 }
 
 sub initialize {
     my $app = shift;
-    $app->terminate('') if $app->no_newline;
-    if ($app->separate) {
-	$app->separate(safe_backslash($app->separate));
+    $app->{terminate} = '' if $app->{no_newline};
+    $app->{separate} = '' if $app->{join};
+    if ($app->{separate}) {
+	$app->{separate} = safe_backslash($app->{separate});
     }
-    $app->separate('') if $app->join;
-    if (defined $app->rgb24) {
-	$Getopt::EX::Colormap::RGB24 = !!$app->rgb24;
+    if (defined $app->{rgb24}) {
+	$Getopt::EX::Colormap::RGB24 = !!$app->{rgb24};
     }
 }
 
@@ -76,7 +80,7 @@ use Getopt::EX::Colormap qw(colorize ansi_code);
 sub retrieve {
     my $app = shift;
     my $count = shift;
-    my $in = $app->params;
+    my $in = $app->{params};
     my @out;
     my @pending;
     my(@style, @effect);
@@ -84,9 +88,15 @@ sub retrieve {
     my $append = sub {
 	push @out, join '', splice(@pending), @_;
     };
+
     while (@$in) {
 	my $arg = shift @$in;
 
+	# -S
+	if ($arg =~ /^-S$/) {
+	    unshift @style, [ \&ansi_code ];
+	    next;
+	}
 	# -c, -C
 	if ($arg =~ /^-([cC])(.+)?$/) {
 	    my $target = $1 eq 'c' ? \@effect : \@style;
@@ -107,25 +117,27 @@ sub retrieve {
 	}
 
 	#
-	# -r     : raw data
-	# -s, -z : ansi sequence
+	# -s, -i, -a : ANSI sequence
 	#
-	if ($arg =~ /^-([szr])(.+)?$/) {
+	if ($arg =~ /^-([sia])(.+)?$/) {
 	    my $opt = $1;
 	    my $text = $2 // shift(@$in) // die "Not enough argument.\n";
-	    my $data = $opt eq 'r' ? safe_backslash($text) : ansi_code($text);
-	    if (@out == 0 or $opt eq 's') {
-		push @pending, $data;
+	    my $code = ansi_code($text);
+	    if ($opt eq 's') {
+		$arg = $code;
 	    } else {
-		$out[-1] .= $data;
+		if (@out == 0 or $opt eq 'i') {
+		    push @pending, $code;
+		} else {
+		    $out[-1] .= $code;
+		}
+		next;
 	    }
-	    next;
 	}
-
 	#
 	# -f : format
 	#
-	if ($arg =~ /^-f(.+)?$/) {
+	elsif ($arg =~ /^-f(.+)?$/) {
 	    my($format) = defined $1 ? safe_backslash($1) : $app->retrieve(1);
 	    my $n = sum map {
 		{ '%' => 0, '*' => 2, '*.*' => 3 }->{$_} // 1
@@ -136,7 +148,7 @@ sub retrieve {
 	# normal string argument
 	#
 	else {
-	    if ($app->escape) {
+	    if ($app->{escape}) {
 		$arg = safe_backslash($arg);
 	    }
 	}
